import numpy as np

def gammaEncode8bit(x, gamma):
    scale = 255
    return int(scale * np.power(x / scale, gamma))

def gammaEncode16bit(x, gamma):
    scale = 65535
    return int(scale * np.power(x / scale, gamma))

def gammaDecode8bit(x, gamma):
    scale = 255
    return int(scale * np.power(x / scale, 1.0/gamma))

def gammaDecode16bit(x, gamma):
    scale = 65535
    return int(scale * np.power(x / scale, 1.0/gamma))

# Generate lookup table for uint16 image encoded at gamma
def gammaLookUpTableUint16(gamma):
    return np.array([gammaDecode16bit(i, gamma=gamma) for i in range(65536)])

# Generate lookup table for uint8 image encoded at gamma
def gammaLookUpTableUint8(gamma):
    return np.array([gammaDecode8bit(i, gamma=gamma) for i in range(256)])

def gammaCorrectImg(img, lut):
    # Use given lookup table to remap img pixels
    return lut.take(img)


from urllib.parse import urlencode, quote  # took out urlparse
from urllib.request import urlopen, Request
from urllib.error import HTTPError

from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart
from email.mime.application import MIMEApplication

from email.encoders import encode_noop

import json


def json2python(data):
    try:
        return json.loads(data)
    except json.JSONDecodeError:
        pass
    return None


python2json = json.dumps


class MalformedResponse(Exception):
    pass


class RequestError(Exception):
    pass


class Client(object):
    default_url = 'http://nova.astrometry.net/api/'

    def __init__(self,
                 apiurl=default_url, tracer=print, trace=False):
        self.session = None
        self.apiurl = apiurl
        self.trace = trace
        self.tracer = tracer   # Ultimately I will use showMsg(msg)

    def get_url(self, service):
        return self.apiurl + service

    # def send_request(self, service, args={}, file_args=None):
    def send_request(self, service, args=None, file_args=None):
        if args is None:
            args = {}
        if self.session is not None:
            args.update({'session': self.session})
        if self.trace:
            self.tracer(f'Python: {args}')
        json_blob = python2json(args)
        if self.trace:
            self.tracer(f'Sending json: {json_blob}')
        url = self.get_url(service)
        if self.trace:
            self.tracer(f'Sending to URL: {url}')

        # If we're sending a file, format a multipart/form-data
        if file_args is not None:
            # Make a custom generator to format it the way we need.
            from io import BytesIO
            # py3
            from email.generator import BytesGenerator as TheGenerator
            # # py2
            # from email.generator import Generator as TheGenerator

            m1 = MIMEBase('text', 'plain')
            m1.add_header('Content-disposition',
                          'form-data; name="request-json"')
            m1.set_payload(json_blob)
            m2 = MIMEApplication(file_args[1], 'octet-stream', encode_noop)
            m2.add_header('Content-disposition',
                          'form-data; name="file"; filename="%s"' % file_args[0])
            mp = MIMEMultipart('form-data', None, [m1, m2])

            class MyGenerator(TheGenerator):
                def __init__(self, fpath, root=True):
                    # don't try to use super() here; in py2 Generator is not a
                    # new-style class.  Yuck.
                    TheGenerator.__init__(self, fpath, mangle_from_=False,
                                          maxheaderlen=0)
                    self.root = root

                def _write_headers(self, msg):
                    # We don't want to write the top-level headers;
                    # they go into Request(headers) instead.
                    if self.root:
                        return
                    # We need to use \r\n line-terminator, but Generator
                    # doesn't provide the flexibility to override, so we
                    # have to copy-n-paste-n-modify.
                    for h, v in msg.items():
                        self._fp.write(('%s: %s\r\n' % (h, v)).encode())
                    # A blank line always separates headers from body
                    self._fp.write('\r\n'.encode())

                # The _write_multipart method calls "clone" for the
                # subparts.  We hijack that, setting root=False
                def clone(self, fpath):
                    return MyGenerator(fpath, root=False)

            fp = BytesIO()
            g = MyGenerator(fp)
            g.flatten(mp)
            data = fp.getvalue()
            headers = {'Content-type': mp.get('Content-type')}

        else:
            # Else send x-www-form-encoded
            data = {'request-json': json_blob}
            if self.trace:
                self.tracer(f'Sending form data: {data}')
            data = urlencode(data)
            data = data.encode('utf-8')
            if self.trace:
                self.tracer(f'Sending data: {data}')
            headers = {}

        request = Request(url=url, headers=headers, data=data)

        try:
            f = urlopen(request)
            txt = f.read()
            if self.trace:
                self.tracer(f'Got json: {txt}')
            result = json2python(txt)
            if self.trace:
                self.tracer(f'Got result: {result}')
            stat = result.get('status')
            if self.trace:
                self.tracer(f'Got status: {stat}')
            if stat == 'error':
                errstr = result.get('errormessage', '(none)')
                raise RequestError('server error message: ' + errstr)
            return result
        except HTTPError as e:
            self.tracer(f'HTTPError: {e}')
            txt = e.read()
            open('err.html', 'wb').write(txt)
            self.tracer(f'Wrote error text to err.html')

    def login(self, apikey):
        args = {'apikey': apikey}
        result = self.send_request('login', args)
        sess = result.get('session')
        if self.trace:
            self.tracer(f'Got session: {sess}')
        if not sess:
            raise RequestError('no session in result')
        self.session = sess

    def _get_upload_args(self, **kwargs):
        args = {}
        for key, default, typ in [('allow_commercial_use', 'd', str),
                                  ('allow_modifications', 'd', str),
                                  ('publicly_visible', 'y', str),
                                  ('scale_units', None, str),
                                  ('scale_type', None, str),
                                  ('scale_lower', None, float),
                                  ('scale_upper', None, float),
                                  ('scale_est', None, float),
                                  ('scale_err', None, float),
                                  ('center_ra', None, float),
                                  ('center_dec', None, float),
                                  ('parity', None, int),
                                  ('radius', None, float),
                                  ('downsample_factor', None, int),
                                  ('positional_error', None, float),
                                  ('tweak_order', None, int),
                                  ('crpix_center', None, bool),
                                  ('x', None, list),
                                  ('y', None, list),
                                  ]:
            if key in kwargs:
                val = kwargs.pop(key)
                val = typ(val)
                args.update({key: val})
            elif default is not None:
                args.update({key: default})
        if self.trace:
            self.tracer(f'Upload args: {args}')
        return args

    def url_upload(self, url, **kwargs):
        args = dict(url=url)
        args.update(self._get_upload_args(**kwargs))
        result = self.send_request('url_upload', args)
        return result

    def upload(self, fn=None, **kwargs):
        args = self._get_upload_args(**kwargs)
        file_args = None
        if fn is not None:
            try:
                f = open(fn, 'rb')
                file_args = (fn, f.read())
            except IOError:
                if self.trace:
                    self.tracer(f'File {fn} does not exist')
                raise
        return self.send_request('upload', args, file_args)

    def submission_images(self, subid):
        result = self.send_request('submission_images', {'subid': subid})
        return result.get('image_ids')

    # def overlay_plot(self, service, outfn, wcsfn, wcsext=0):
    #     wcs = anutil.Tan(wcsfn, wcsext)
    #     params = dict(crval1 = wcs.crval[0], crval2 = wcs.crval[1],
    #                   crpix1 = wcs.crpix[0], crpix2 = wcs.crpix[1],
    #                   cd11 = wcs.cd[0], cd12 = wcs.cd[1],
    #                   cd21 = wcs.cd[2], cd22 = wcs.cd[3],
    #                   imagew = wcs.imagew, imageh = wcs.imageh)
    #     result = self.send_request(service, {'wcs':params})
    #     if self.trace:
    #         print('Result status:', result['status'])
    #     plotdata = result['plot']
    #     plotdata = base64.b64decode(plotdata)
    #     open(outfn, 'wb').write(plotdata)
    #     if self.trace:
    #         print('Wrote', outfn)

    # def sdss_plot(self, outfn, wcsfn, wcsext=0):
    #     return self.overlay_plot('sdss_image_for_wcs', outfn,
    #                              wcsfn, wcsext)

    # def galex_plot(self, outfn, wcsfn, wcsext=0):
    #     return self.overlay_plot('galex_image_for_wcs', outfn,
    #                              wcsfn, wcsext)

    def myjobs(self):
        result = self.send_request('myjobs/')
        return result['jobs']

    def job_status(self, job_id, justdict=False):
        result = self.send_request('jobs/%s' % job_id)
        if justdict:
            return result
        stat = result.get('status')
        if stat == 'success':
            # result = self.send_request('jobs/%s/calibration' % job_id)
            # print('Calibration:', result)
            # result = self.send_request('jobs/%s/tags' % job_id)
            # print('Tags:', result)
            # result = self.send_request('jobs/%s/machine_tags' % job_id)
            # print('Machine Tags:', result)
            # result = self.send_request('jobs/%s/objects_in_field' % job_id)
            # print('Objects in field:', result)
            # result = self.send_request('jobs/%s/annotations' % job_id)
            # print('Annotations:', result)
            result = self.send_request('jobs/%s/info' % job_id)
            self.tracer(f'Calibration: {result}')

        return stat

    def annotate_data(self, job_id):
        """
        :param job_id: id of job
        :return: return data for annotations
        """
        result = self.send_request('jobs/%s/annotations' % job_id)
        return result

    def sub_status(self, sub_id, justdict=False):
        result = self.send_request('submissions/%s' % sub_id)
        if justdict:
            return result
        return result.get('status')

    def jobs_by_tag(self, tag, exact):
        exact_option = 'exact=yes' if exact else ''
        result = self.send_request(
            'jobs_by_tag?query=%s&%s' % (quote(tag.strip()), exact_option),
            {},
        )
        return result



import subprocess
import json, re
import urllib.request, urllib.error, urllib.parse
from distutils.version import StrictVersion

def getMostRecentVersionOfPyMovieViaJason():

    #!!!!! Many thanks to Kia Getrost for supplying this much improved version of 'getMostRecentVersionOfPymovie'

    # Returns tuple of gotVersion, latestVersion
    # (boolean and version-or-error string)

    pkgName = "pymovie"

    # Do a JSON request to pypi to get latest version:
    url = f"https://pypi.org/pypi/{pkgName}/json"
    text = getUrlAsText(url)
    if text is None:
        return False, "Could not contact pypi.org to check for latest version"

    # Parse the JSON result:
    try:
        data = json.loads(text)
    except ValueError:
        return False, "Could not parse JSON response from pypi.org"

    # Sort versions to get the latest:
    versions = sorted(data["releases"], key=StrictVersion, reverse=True)
    latestVersion = versions[0]

    # Ensure we have a seemingly valid vesrion number:
    if not re.match(r"\d+\.\d+\.\d+", latestVersion):
        return False, f"Garbled version `{latestVersion}' from pypi.org"

    # All is well, return result:
    return True, latestVersion

def getUrlAsText(url):
    # Returns text string of `url', or None on error

    try:
        request = urllib.request.Request(url)
        response = urllib.request.urlopen(request)
    except urllib.error.URLError as exception:
        if hasattr(exception, "reason"):
            print(f"Fetch of `{url}' failed: {exception.reason}")
        elif hasattr(exception, "code"):
            print(f"Fetch of `{url}' failed: returned HTTP code {exception.code}")
        else:
            print(f"Fetch of `{url}' failed: Unknown reason")
        return None
    html = response.read()
    text = html.decode("utf-8")
    return text

def upgradePyMovie(pymovieversion):

    import subprocess

    resp = subprocess.run(['python', '-m', 'pip', 'install', '--user', '--upgrade', pymovieversion],
                          stderr=subprocess.PIPE, stdout=subprocess.PIPE)

    ans = resp.stdout.decode("utf-8").split('\n')

    return ans



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'helpDialog.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(991, 434)
        Dialog.setSizeGripEnabled(True)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.textEdit = QtWidgets.QTextEdit(Dialog)
        self.textEdit.setReadOnly(True)
        self.textEdit.setObjectName("textEdit")
        self.gridLayout.addWidget(self.textEdit, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Help"))
        self.textEdit.setToolTip(_translate("Dialog", "<html><head/><body><p>This dialog box is non-modal, so there is no need to close it in order to continue using the main GUI<br/></p><p>In fact, best practice is to set your preferred size and position and DO NOT close the dialog. When you click on the main GUI, this dialog box may disappear behind the main GUI, but a new right-click will pop it back up at the previous size and position.</p><p>TIP: <span style=\" font-weight:600; color:#fc0107;\">click on labels</span>, particularly ones that label entry boxes.  Because images and entry boxes have an inherent right-click context menu, in order to avoid conflicting with that usage, the right-click help is attached to the label rather than the control itself.</p></body></html>"))
        self.textEdit.setHtml(_translate("Dialog", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'.SF NS Text\'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))





import pyqtgraph as pg
from pyqtgraph.Qt import QtCore, QtGui


class OcrAperture(pg.GraphicsObject):
    """
    This class draws a rectangular area. Right-clicking inside the area will
    raise a custom context menu.
    """

    # Intialize aperture specified by a field box
    def __init__(self, fieldbox, boxnum, position, msgRoutine, templater,
                 jogcontroller, frameview, showcharacter, showtemplates, neededdigits, kiwi=False, samplemenu=True):
        self.samplemenu = samplemenu
        self.kiwiStyle = kiwi
        self.templateWriter = templater             # self.processOcrTemplate()  in main
        self.displayDigitTemplates = showtemplates  # self.showDigitTemplates()  in main
        self.controlAllJogs = jogcontroller         # self.setAllOcrBoxJogging() in main
        self.frameViewReference = frameview         # self.frameView             in main
        self.showCharacter = showcharacter          # self.showOcrCharacter()    in main
        self.neededDigits = neededdigits            # self.needDigits()          in main
        self.boxnum = boxnum
        self.msgRoutine = msgRoutine                # self.showMsg() in main
        self.joggable = False
        self.position = position  # 'upper'  or 'lower'
        self.pen = pg.mkPen('r')
        self.color = 'red'
        xL, xR, yU, yL = fieldbox
        self.x0 = xL
        self.y0 = yU
        self.xsize = xR - xL
        self.ysize = yL - yU
        self.menu = None

        # note that the use of super() is often avoided because Qt does not
        # allow to inherit from multiple QObject subclasses.
        pg.GraphicsObject.__init__(self)

    def setBox(self, fieldbox):
        xL, xR, yU, yL = fieldbox
        self.x0 = xL
        self.y0 = yU
        self.xsize = xR - xL
        self.ysize = yL - yU

    def getBox(self):  # Return 'internal' box
        xR = self.xsize + self.x0
        yL = self.ysize + self.y0
        return self.x0, xR - 1, self.y0, yL - 1  #(xL, xR, yU, yL)

    # All graphics items must have boundingRect() defined.
    def boundingRect(self):
        return QtCore.QRectF(self.x0, self.y0, self.xsize, self.ysize)

    # All graphics items must have paint() defined.
    def paint(self, p, *args):
        _ = args # unused parameter
        p.setPen(self.pen)
        if self.joggable:
            p.setPen(pg.mkPen('y'))
        else:
            p.setPen(pg.mkPen('r'))
        if not self.kiwiStyle:
            p.drawRect(self.boundingRect())
        else:
            # p.setPen(pg.mkPen('y'))
            # p.setPen(pg.mkPen('r'))
            dx = 5
            x0 = self.x0 + dx
            y0 = self.y0
            x1 = x0 + self.xsize
            y1 = y0
            x2 = self.x0 - dx
            y2 = self.y0 + self.ysize
            x3 = x2 + self.xsize
            y3 = y2
            p.drawLine(x0, y0, x1, y1) # Top line
            p.drawLine(x0, y0, x2, y2) # Left line
            p.drawLine(x1, y1, x3, y3) # Right line
            p.drawLine(x2, y2, x3, y3) # Bottom line

    # On right-click, raise the context menu
    def mouseClickEvent(self, ev):
        if ev.button() == QtCore.Qt.RightButton:
            if self.raiseContextMenu(ev):
                ev.accept()

    def raiseContextMenu(self, ev):
        menu = self.getContextMenus()

        pos = ev.screenPos()
        menu.popup(QtCore.QPoint(pos.x(), pos.y()))
        return True

    def getContextMenus(self, event=None):
        # if self.menu is None:
        self.menu = QtGui.QMenu()
        # self.menu.setTitle(self.name + " options..")  # This appears to do nothing

        setshowprops = QtGui.QAction("Show properties", self.menu)
        setshowprops.triggered.connect(self.showProps)
        self.menu.addAction(setshowprops)

        self.menu.addSeparator()

        setjogon = QtGui.QAction("Enable jogging", self.menu)
        setjogon.triggered.connect(self.enableJogging)
        self.menu.addAction(setjogon)

        setjogoff = QtGui.QAction("Disable jogging", self.menu)
        setjogoff.triggered.connect(self.disableJogging)
        self.menu.addAction(setjogoff)

        self.menu.addSeparator()

        setupperjogs = QtGui.QAction("Enable jogging for upper boxes", self.menu)
        setupperjogs.triggered.connect(self.enableUpperJogging)
        self.menu.addAction(setupperjogs)

        setlowerjogs = QtGui.QAction("Enable jogging for lower boxes", self.menu)
        setlowerjogs.triggered.connect(self.enableLowerJogging)
        self.menu.addAction(setlowerjogs)

        setalljogs = QtGui.QAction("Enable jogging for all boxes", self.menu)
        setalljogs.triggered.connect(self.enableAllJogging)
        self.menu.addAction(setalljogs)

        self.menu.addSeparator()

        clearupperjogs = QtGui.QAction("Disable jogging for upper boxes", self.menu)
        clearupperjogs.triggered.connect(self.disableUpperJogging)
        self.menu.addAction(clearupperjogs)

        clearlowerjogs = QtGui.QAction("Disable jogging for lower boxes", self.menu)
        clearlowerjogs.triggered.connect(self.disableLowerJogging)
        self.menu.addAction(clearlowerjogs)

        clearalljogs = QtGui.QAction("Disable jogging for all boxes", self.menu)
        clearalljogs.triggered.connect(self.disableAllJogging)
        self.menu.addAction(clearalljogs)


        self.menu.addSeparator()

        showdigits = QtGui.QAction('show model digits', self.menu)
        showdigits.triggered.connect(self.showDigits)
        self.menu.addAction(showdigits)

        retraindigits = QtGui.QAction('retrain model digits', self.menu)
        retraindigits.triggered.connect(self.retrainDigits)
        self.menu.addAction(retraindigits)

        if self.samplemenu:

            self.menu.addSeparator()

            needed = self.neededDigits()

            if needed[0]:
                set0 = QtGui.QAction("record 0", self.menu)
                set0.triggered.connect(self.write0)
                self.menu.addAction(set0)

            if needed[1]:
                set1 = QtGui.QAction("record 1", self.menu)
                set1.triggered.connect(self.write1)
                self.menu.addAction(set1)

            if needed[2]:
                set2 = QtGui.QAction("record 2", self.menu)
                set2.triggered.connect(self.write2)
                self.menu.addAction(set2)

            if needed[3]:
                set3 = QtGui.QAction("record 3", self.menu)
                set3.triggered.connect(self.write3)
                self.menu.addAction(set3)

            if needed[4]:
                set4 = QtGui.QAction("record 4", self.menu)
                set4.triggered.connect(self.write4)
                self.menu.addAction(set4)

            if needed[5]:
                set5 = QtGui.QAction("record 5", self.menu)
                set5.triggered.connect(self.write5)
                self.menu.addAction(set5)

            if needed[6]:
                set6 = QtGui.QAction("record 6", self.menu)
                set6.triggered.connect(self.write6)
                self.menu.addAction(set6)

            if needed[7]:
                set7 = QtGui.QAction("record 7", self.menu)
                set7.triggered.connect(self.write7)
                self.menu.addAction(set7)

            if needed[8]:
                set8 = QtGui.QAction("record 8", self.menu)
                set8.triggered.connect(self.write8)
                self.menu.addAction(set8)

            if needed[9]:
                set9 = QtGui.QAction("record 9", self.menu)
                set9.triggered.connect(self.write9)
                self.menu.addAction(set9)

        return self.menu

    def showDigits(self):
        self.displayDigitTemplates()

    def retrainDigits(self):
        self.displayDigitTemplates(retrain=True)

    def disableUpperJogging(self):
        self.controlAllJogs(enable=False, position='upper')
        self.frameViewReference.getView().update()

    def disableLowerJogging(self):
        self.controlAllJogs(enable=False, position='lower')
        self.frameViewReference.getView().update()

    def disableAllJogging(self):
        self.disableLowerJogging()
        self.disableUpperJogging()

    def enableUpperJogging(self):
        self.controlAllJogs(enable=True, position='upper')
        self.frameViewReference.getView().update()

    def enableLowerJogging(self):
        self.controlAllJogs(enable=True, position='lower')
        self.frameViewReference.getView().update()

    def enableAllJogging(self):
        self.enableLowerJogging()
        self.enableUpperJogging()

    def write0(self):
        self.templateWriter(0, self.getBox())

    def write1(self):
        self.templateWriter(1, self.getBox())

    def write2(self):
        self.templateWriter(2, self.getBox())

    def write3(self):
        self.templateWriter(3, self.getBox())

    def write4(self):
        self.templateWriter(4, self.getBox())

    def write5(self):
        self.templateWriter(5, self.getBox())

    def write6(self):
        self.templateWriter(6, self.getBox())

    def write7(self):
        self.templateWriter(7, self.getBox())

    def write8(self):
        self.templateWriter(8, self.getBox())

    def write9(self):
        self.templateWriter(9, self.getBox())

    def showProps(self):
        msg = f'ocrbox: {self.position}-{self.boxnum}   box is: {self.getBox()}'
        if self.joggable:
            msg += f' (jogging enabled)'
        self.msgRoutine(msg=msg)
        self.showCharacter(self.getBox())

    def enableJogging(self):
        self.joggable = True
        self.pen = pg.mkPen('y')
        self.frameViewReference.getView().update()

    def disableJogging(self):
        self.joggable = False
        self.pen = pg.mkPen('r')
        self.frameViewReference.getView().update()



# play-instrospection.py

from typing import Dict, Any, NewType

UserId = NewType('UserId', int)

bobs_id = UserId(52)


def obj_lister(object_source: Dict[str, Any], *types_to_report):
    count_of_others = 0
    line_num = 1
    for key, obj in object_source.items():
        if type(obj) in types_to_report:
            print(f'{line_num:4d} {key}: {type(obj).__name__}')
        else:
            print(f'{line_num:4d} ???      {key}: {type(obj).__name__}')
            count_of_others += 1
        line_num += 1
    print(f'\nFound {count_of_others} other objects')

bob = 78
judy = 3.14
NoneType = type(None)
FuncType = type(obj_lister)
BuiltInType = type(abs)

print(len(vars()))
obj_lister(vars(), str, int, float, dict, type, NoneType, FuncType, BuiltInType)
# g: Dict[str, Any] = vars().copy()
# for key, obj in g.items():
#     print(f'{key} {type(obj)}')


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'selectProfile.ui'
#
# Created by: PyQt5 UI code generator 5.12.1
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(868, 322)
        self.gridLayout = QtWidgets.QGridLayout(Dialog)
        self.gridLayout.setObjectName("gridLayout")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.selectionTable = QtWidgets.QTableWidget(Dialog)
        self.selectionTable.setRowCount(0)
        self.selectionTable.setColumnCount(1)
        self.selectionTable.setObjectName("selectionTable")
        item = QtWidgets.QTableWidgetItem()
        self.selectionTable.setHorizontalHeaderItem(0, item)
        self.selectionTable.horizontalHeader().setStretchLastSection(True)
        self.horizontalLayout.addWidget(self.selectionTable)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.loadButton = QtWidgets.QPushButton(Dialog)
        self.loadButton.setObjectName("loadButton")
        self.verticalLayout.addWidget(self.loadButton)
        self.deleteButton = QtWidgets.QPushButton(Dialog)
        self.deleteButton.setObjectName("deleteButton")
        self.verticalLayout.addWidget(self.deleteButton)
        self.exitButton = QtWidgets.QPushButton(Dialog)
        self.exitButton.setDefault(True)
        self.exitButton.setObjectName("exitButton")
        self.verticalLayout.addWidget(self.exitButton)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.profileNameEdit = QtWidgets.QLineEdit(Dialog)
        self.profileNameEdit.setObjectName("profileNameEdit")
        self.horizontalLayout_2.addWidget(self.profileNameEdit)
        self.addProfileButton = QtWidgets.QPushButton(Dialog)
        self.addProfileButton.setObjectName("addProfileButton")
        self.horizontalLayout_2.addWidget(self.addProfileButton)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.gridLayout.addLayout(self.verticalLayout_2, 0, 0, 1, 1)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Select custom OCR profile"))
        item = self.selectionTable.horizontalHeaderItem(0)
        item.setText(_translate("Dialog", "OCR profile description"))
        self.loadButton.setText(_translate("Dialog", "Load selected profile"))
        self.deleteButton.setText(_translate("Dialog", "Delete selected profile"))
        self.exitButton.setText(_translate("Dialog", "Exit"))
        self.label.setText(_translate("Dialog", "New profile name:"))
        self.addProfileButton.setText(_translate("Dialog", "Add current ocr profile to list"))





import subprocess
import os
import shlex


# returns the full path of the file "pointed to" by the alias
def resolve_osx_alias(path):        # single file/path name
    checkpath = os.path.abspath(path)       # osascript needs absolute paths
    # Next several lines are AppleScript
    line_1='tell application "Finder"'
    line_2='set theItem to (POSIX file "'+checkpath+'") as alias'
    line_3='if the kind of theItem is "alias" then'
    line_4='   get the posix path of (original item of theItem as text)'
    line_5='else'
    line_6='return "'+checkpath+'"'
    line_7 ='end if'
    line_8 ='end tell'
    cmd = "osascript -e '"+line_1+"' -e '"+line_2+"' -e '"+line_3+"' -e '"+line_4+"' -e '"+line_5+"' -e '"+line_6+"' -e '"+line_7+"' -e '"+line_8+"'"
    # shlex splits cmd up appropriately so we can call subprocess.Popen with shell=False (better security)
    args = shlex.split(cmd)
    p = subprocess.Popen(args, shell=False, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    retval = p.wait()
    if retval == 0:
        line = p.stdout.readlines()[0]
        source = line.decode('UTF-8').replace('\n','')
        # if (convert):
        #     os.remove(checkpath)
        #     os.symlink(source, checkpath)
    else:
        print('resolve_osx_aliases: Error: subprocess returned non-zero exit code '+str(retval))
        source = ''
    return source


def create_osx_alias_in_dir(file_path, dir_path):
    theFile = os.path.abspath(file_path)   # osascript needs absolute paths
    theDir = os.path.abspath(dir_path)
    line1 = 'tell application "Finder"'
    line2 = '  make new alias file at (POSIX file "'+theDir+'") to (POSIX file "'+theFile+'" as alias)'
    line3 = 'end tell'
    cmd = "osascript -e '"+line1+"' -e '"+line2+"' -e '"+line3+"'"
    args = shlex.split(cmd)
    p = subprocess.Popen(args, shell=False, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
    retval = p.wait()
    if retval == 0:
        line = p.stdout.readlines()[0]
        source = line.decode('UTF-8').replace('\n', '')
        return True, theFile, theDir, retval, source
    else:
        line = p.stdout.readlines()[0]
        source = line.decode('UTF-8').replace('\n', '')
        return False, theFile, theDir, retval, source

